@@ -31,7 +31,6 @@ class GKOProblem(AbstractKOProblem):
     def _build_target_list(self):
         print("Building modification target list.")
         genes = set(self.simulator.genes)
-        print("Computing essential genes.")
         essential = set(self.simulator.essential_genes())
         transport = set(self.simulator.get_transport_genes())
         target = genes - essential - transport
@@ -91,6 +90,8 @@ class GOUProblem(AbstractOUProblem):
         target = genes - transport
         if self.non_target:
             target = target - set(self.non_target)
+        if self._partial_solution:
+            target = target - set(self._partial_solution.keys())
         target = list(target)
         self._trg_list = target
 
