@@ -4,10 +4,35 @@ import numpy as np
 from ..simulation import get_simulator
 from ..simulation.simulation import Simulator
 
-METABOLITE = 'metabolite'
-REACTION = 'reaction'
-REV = 'rev'
-IRREV = 'irrev'
+METABOLITE = 'METABOLITE'
+REACTION = 'REACTION    '
+REV = 'REV'
+IRREV = 'IRREV'
+
+COFACT = ['H', 
+          'Mg',
+          'Mn',
+          'H',
+          'Zn',
+          'CO2',
+          'H20',
+          'HO4P',
+          'C21H26N7O17P3',      # NADPH
+          'C21H25N7O17P3',      # NADP
+          'C21H27N7O14P2',      # NAD
+          'C10H12N5O13P3',      # ATP
+          'C10H12N5O10P2',      # ADP
+          'C27H33N9O15P2',      # FAD
+          'C21H36N7O16P3S',     # CoA
+          'C12H19N4O7P2S',      # TPP
+          'C8H10NO6P',          # P5P
+          'C63H91CoN13O14P',    # Vitamin B12
+          'C63H88CoN14O14P',    # Vitamin B12
+          'C6H8O6',             # Vitamin C
+          'C10H16N2O3S',        # Vitamin B7
+          'C19H23N7O6',         # THFA
+          'HO7P2',              # ppi
+         ]
 
 
 def create_metabolic_graph(model, directed=True, carbon=True, reactions=None, remove=[], edges_labels=False, biomass=False, metabolites=False):
@@ -44,7 +69,7 @@ def create_metabolic_graph(model, directed=True, carbon=True, reactions=None, re
     for r in reactions:
         the_metabolites = container.get_reaction_metabolites(r)
         for m in the_metabolites:
-            if m in remove:
+            if m in remove or container.get_metabolite(m)['formula'] in COFACT:
                 continue
             if carbon and 'C' not in container.metabolite_elements(m).keys():
                 continue
